{ Test.pas    v2.0  Copyright 5/1994  Adolfo Di Mare }

{$I ADT-ADH.inc}

UNIT Test;
{ RESULTADO
  Esta unidad le permite al programador desarrollar y
  probar programas de manera que pueda centrarse en
  sto sin necesidad de poner atencin al manejo del
  men para los casos de prueba.

  La informacin sobre cmo debe se usar esta unidad
  se encuentra detallada en el archivo Test.doc. }

{ REQUIERE
  El mximo de opciones que pueden ser manejadas por
  esta unidad es 26. Todos los procedimientos que se
  incluyan en las rutinas de esta unidad deben ser
  compilados como procedimientos que se invocan con
  llamados lejanos (opcin $F+ del compilador). }

{ PROGRAMADOR
  Gustavo Alonso Sanabria Sanabria.
  Universidad de Costa Rica.
  Octubre de 1989. }

{ MODIFICACIONES

  OCT-89  Gustvo A. Sanabria S:
  - Se agreg el procedimiento Inter_Menu, el cual se
    encarga de ejecutar en una rutina despus de pedir
    la opcin del men pero antes de entrar a la ocin
    seleccionada.

  - Se agreg el procedimiento Flush_Keyboard() que
    limpia el buffer del teclado.

  - La lista de opciones ahora contiene un registro
    con dos campos: op_proc y msg.

  - El procedimiento Say lo que hace es llenar todos
    los campos con el mensaje de la opcin, en lugar
    de tener que estar llamando a las opciones cada
    vez que se escribe el men.

  NOV-89 Gustvo A. Sanabria S:
  - Se agreg el procedimiento privado Print_Msg, para
    la impresin de los mensajes de las opciones.
  - Se eliminaron las banderas booleanas. }

{[])===========([]}
{[]} INTERFACE {[]}
{[])===========([]}

CONST
  { Lineas permitidas para subrayado del ttulo }
  No_line   = #0;         { ''  (sin subrrayar) }
  Dbl_Line  = #205;       { ''                 }
  Sngl_Line = #196;       { ''                 }

TYPE
  { procedimientos de opciones }
  Option_Proc = PROCEDURE;

  { largo mximo de la tira para ttulos }
  { y para mensajes de opciones.         }
  Str80_T = STRING[80];

{ Procedimietos para manejo el men }

PROCEDURE Title        (tit   : STRING; y_pos: BYTE; u: CHAR);
PROCEDURE Say          (op_msg: STRING);
FUNCTION  Display      {>>>>>}: BOOLEAN;

PROCEDURE First_Proc   (proc  : Option_Proc);
PROCEDURE Last_Proc    (proc  : Option_Proc);
PROCEDURE Pre_Menu     (proc  : Option_Proc);
PROCEDURE Inter_Menu   (proc  : Option_Proc);
PROCEDURE Post_Menu    (proc  : Option_Proc);
PROCEDURE Force_Option (proc  : Option_Proc; n: WORD);
PROCEDURE Add_Option   (proc  : Option_Proc);

FUNCTION  Option_Str   {>>>>>}: STRING;

PROCEDURE GoWork;

{ Procedimientos miscelneos }
PROCEDURE KeyDelay(secDelay: INTEGER);
PROCEDURE KeyDelayPlus(VAR delay: WORD; delta: WORD);
PROCEDURE Beep;
PROCEDURE Flush_Keyboard;
FUNCTION  ReadKey: CHAR;
FUNCTION  Add_Commas(n : LONGINT): STRING;
PROCEDURE Lea_Elem (VAR c : CHAR; VAR i : INTEGER);
PROCEDURE BOOM(x,y: WORD);
PROCEDURE WriteXY(x,y: WORD; s: STRING);
FUNCTION  ExistFile(name : STRING): BOOLEAN;

{[])================([]}
{[]} IMPLEMENTATION {[]}
{[])================([]}

USES
  Crt, Dos;

CONST
  Esc = #27; { Tecla ESCape }

TYPE
  OptionRec = RECORD
    op_proc : Option_Proc; { rutina de la opcin  }
    msg     : Str80_T;     { mensaje de la opcin }
  END; { OptionRec }
  Option_List = ARRAY [PRED('A')..'Z'] OF OptionRec;

VAR
  { variables globales usadas por esta unidad }
  GLOBAL : RECORD
    option      : CHAR;
    last_option : CHAR;
    op_list     : Option_List;
    on_display  : BOOLEAN;
    opt_Init    : Option_Proc;
    opt_Done    : Option_Proc;
    opt_preMenu : Option_Proc;
    opt_preOpc  : Option_Proc;
    opt_postOpc : Option_Proc;
    opt_forced  : Option_Proc;
    menu_tit    : STRING;
    tit_col     : BYTE;
    tit_row     : BYTE;
    op_col      : BYTE;
    force_cnt   : INTEGER;
    under       : CHAR;
{ Descripcin de estas variables globales:
  option:
    opcin actual del men que es ejecutada.
  last_option:
    seala cual es la ltima opcin en el men.
  op_list:
    lista con las rutinas de las opciones del men y
    con los mensajes que estos despliegan.
  op_forced:
    indica si alguna opcin del men est siendo
    forzada para ejecutarse.
  op_first:
    indica si hay alguna rutina que se ha de ejecutar
    antes de trabajar con el men.
  op_last:
    indica si hay alguna rutina que ha de ejecutase
    despus de trabajar con el men.
  op_pre:
    indica si hay una rutina que ha de ejecutarse
    antes de entrar cada ciclo en el men.
  op_inter:
    indica si hay una rutina que se ejecuta despus de
    pedir una opcin en el men pero antes de
    ejecutarla.
  op_post:
    indica si hay una rutina que deba relizarse
    despus de salir del men en cada ciclo.
  on_display:
    indica si se estn desplegando el ttulo del men
    o algn mensaje de las opciones. Su valor puede se
    conocido mediante la funcin Display, pero nunca
    modificado desde afuera.
  opt_Init:
    guarda el procedimiento que ha de ser llamado
    antes de trabajar con el men.
  opt_Done:
    guarda el procedimiento que se llama despus de
    terminado el trabajo con el men.
  opt_preMenu:
    guarda la subrutina que tiene que ejecutarse antes
    de desplegar el men en cada ciclo.
  opt_preOpc:
    guarda la subrutina que se ejecuta despus de
    pedir una opcin pero antes de ejecutarla.
  opt_postOpc:
    guarda la rutina que se llama despus de la salida
    de una opcin dada en el men.
  opt_forced:
    guarda la rutina que se fuerza a ejecutar un
    nmero dado de veces.
  menu_tit:
    ttulo del men.
  tit_col:
    guarda la columna del ttulo.
  tit_row:
    guarda la fila donde se debe colocar el men.
  under:
    subrayado del ttulo.
  force_cnt:
    cantidad de veces que se ejecuta la opcin forzada.
} END; { GLOBAL }

PROCEDURE InitGlobal;  { IMPORT } { Sanabria }
{ RESULTADO
  Inicializa las variables globales }
VAR
  ch : CHAR;
BEGIN { InitGlobal }
  FOR ch := PRED('A') TO 'Z' DO BEGIN
    GLOBAL.op_list[ch].msg      := '';
    @GLOBAL.op_list[ch].op_proc := NIL;
  END;

  GLOBAL.op_col      := 40;    { mensajes centrados }
  GLOBAL.option      := 'A';
  GLOBAL.last_option := #64;
  GLOBAL.force_cnt   := 0;

  @GLOBAL.opt_forced := NIL;
  @GLOBAL.opt_Init   := NIL;
  @GLOBAL.opt_Done   := NIL;
  @GLOBAL.opt_preOpc := NIL;
  @GLOBAL.opt_preMenu:= NIL;
  @GLOBAL.opt_postOpc:= NIL;

  GLOBAL.on_display  := TRUE;
  GLOBAL.menu_tit    := '';
  GLOBAL.tit_col     := 40;  { centrado en la fila 1 }
  GLOBAL.tit_row     := 1;
  GLOBAL.under       := No_line;
END;  { InitGlobal }

PROCEDURE Title(         { EXPORT }     { Sanabria }
  tit   : STRING;        { ttulo del men }
  y_pos : BYTE;          { # file }
  u     : CHAR           { tipo de subrayado }
);
{ RESULTADO
  Almacena el ttulo para el men principal. 
  - El ttulo ser desplegaod en la fila "y_pos",y 
    estar subrrayado con el caracter "u". }
{ REQUIERE
  underline DEBE ser alguno de los caracteres
  definidos en la INTERFACE                   }
BEGIN { Title }
  IF NOT (u IN [No_line, Dbl_Line, Sngl_Line]) THEN BEGIN
    RunError(201); { Range check error }
  END;
  GLOBAL.menu_tit := tit;
  GLOBAL.tit_col  := 40 - (Ord(tit[0]) DIV 2);
  GLOBAL.tit_row  := y_pos;
  GLOBAL.under    := u;
END;  { Title }

PROCEDURE Say( { EXPORT } { Sanabria }
  op_msg : STRING
);
{ RESULTADO
  Guarda el mensaje para la opcin y calcula
  donde desplegar centrada la ms larga y las
  dems. }
VAR
  center : BYTE; { columna para centrar el mensaje }
BEGIN { Say }
  GLOBAL.op_list[GLOBAL.option].msg := op_msg;

  center := 40 - (Ord(op_msg[0]) DIV 2);
  IF center < GLOBAL.op_col THEN BEGIN
    GLOBAL.op_col := center;
  END;
END;  { Say }

FUNCTION Display
{>>>>} : BOOLEAN;
{ RESULTADO
  Chequea la variable on_display para despliege del
  ttulo del men y de los mensajes de las opciones }
BEGIN { Display }
  Display := GLOBAL.on_display;
END;  { Display }

PROCEDURE First_Proc( { EXPORT } { Sanabria }
 {+} proc : Option_Proc
);
{ RESULTADO
  Almacena una rutina que ha de ejecutarse UNICAMENTE
  antes de entrar al men la primera vez.             }
BEGIN { First_Proc }
  GLOBAL.opt_Init := proc;
END;  { First_Proc }

PROCEDURE Last_Proc(  { EXPORT } { Sanabria }
  proc : Option_Proc
);
{ RESULTADO
  Almacena una rutina que ha de ejecutarse UNICAMENTE
  al salir del men por ultima                        }
BEGIN { Last_Proc }
  GLOBAL.opt_Done := proc;
END;  { Last_Proc }

PROCEDURE Pre_Menu( { EXPORT } { Sanabria }
 {+} proc : Option_Proc
);
{ RESULTADO
  Ejecuta la rutina proc en cada ciclo antes
  de entrar en el men                       }
BEGIN { Pre_Menu }
  GLOBAL.opt_preMenu := proc;
END;  { Pre_Menu }

PROCEDURE Inter_Menu( { EXPORT } { Sanabria }
 {+} proc : Option_Proc
);
{ RESULTADO
  Ejecuta la rutina proc en cada ciclo antes
  de despus de salir del men pero antes de
  entrar a ejecutar la opcin escogida.     }
BEGIN { Inter_Menu }
  GLOBAL.opt_preOpc := proc;
END;  { Inter_Menu }

PROCEDURE Post_Menu( { EXPORT } { Sanabria }
 {+} proc : Option_Proc
);
{ RESULTADO
  Ejecuta la rutina proc en cada ciclo despus
  de salir del men                            }
BEGIN { Post_Menu }
  GLOBAL.opt_postOpc := proc;
END;  { Post_Menu }

PROCEDURE Force_Option(  { EXPORT }     { GAS }
  {+} proc : Option_Proc;
  {+} n    : WORD
);
{ RESULTADO
  Ejecuta "n" veces el procedimiento "proc" antes de
  desplegar por primera vez el men de opciones.
  - "proc" debe ser un procedimiento lejano que no
    usa argumentos. }
BEGIN { Force_Option }
  IF n < 1 THEN BEGIN
    ClrScr;
    WriteLn;
    WriteLn('Force_Option(',n:1,') ==> No se fuerza opcin');
    WriteLn;
    KeyDelay(5);
    { RunError(201);  Range check error }
  END
  ELSE BEGIN
    GLOBAL.opt_forced := proc;
    GLOBAL.force_cnt  := n;
    GLOBAL.on_display := FALSE;
  END;
END;  { Force_Option }

PROCEDURE Add_Option( { EXPORT } { Sanabria }
  {+} proc : Option_Proc
);
{ RESULTADO
  Agrega una  nueva rutina al men.  A la primera se le
  asigna la opcin 'A', a la segunda la 'B' y as hasta
  la nmero 26 ('Z').                                   }
{ REQUIERE
  La cantidad de opciones debe ser menor o igual que 26 }
BEGIN { Add_Option }
  IF GLOBAL.last_option = 'Z' THEN BEGIN
    RunError(201); { Range check error }
  END;
  INC(GLOBAL.last_option);
  GLOBAL.op_list[GLOBAL.last_option].op_proc := proc;
END;  { Add_Option }

FUNCTION  Option_Str     { EXPORT }     { ADH }
  {>>>>>>>>>>>>>>>>}: STRING;
{ RESULTADO
  Retorna la hilera que se us al desplegar en el men
  la opcin que ha sido seleccionada por el operador
  del programa. }
BEGIN { Option_Str }
  Option_Str :=
    GLOBAL.op_list[GLOBAL.option].msg;
END;  { Option_Str }

PROCEDURE Print_Msgs; { IMPORT } { Sanabria }
{ RESULTADO
  Imprime los mensajes de las opciones en dos formatos:
  centradas de  acuerdo con el mensaje ms  extenso o a
  dos columnas cuando hay demasiadas opciones  o cuando
  el men est queda demasiado  cargado hacia  abajo en
  la pantalla, lo que desmejora la presentacin.        }
VAR
  col, row,
  max_op,
  lineas   : BYTE; { imprime a doble columna si no hay campo }
BEGIN { Print_Msgs }
  IF (23 - GLOBAL.tit_row > Ord(GLOBAL.last_option) - 65)
  THEN BEGIN

    { si hay suficiente espacio se centran la opciones }
    FOR GLOBAL.option := 'A' TO GLOBAL.last_option DO BEGIN
      GotoXY(
        GLOBAL.op_col,
        ORD(GLOBAL.option) - 63 + GLOBAL.tit_row);
      Write(
        GLOBAL.option, '. ',
        GLOBAL.op_list[GLOBAL.option].msg);
    END;
  END
  ELSE BEGIN
    { si no hay suficiente espacio lo imprime:
      (columna 2)             (columna 42)
      A. Opcin 1             C. Opcin 3
      B. Opcin 2             D. Opcin 4
      ...
      (los mensajes son truncados si son muy largos) }

    col := 2;
    row := GLOBAL.tit_row + 2;
    max_op := Ord(GLOBAL.last_option) - Ord('A');
    lineas := 0;
    FOR GLOBAL.option := 'A' TO GLOBAL.last_option DO BEGIN
      GotoXY(col,row);
      Write(
          GLOBAL.option
        + '. '
        + GLOBAL.op_list[GLOBAL.option].msg);

      IF lineas <> (max_op div 2) THEN BEGIN
        { Verifica que la cantidad  }
        { de lineas por columna sea }
        { la misma.                 }
        INC(row);
      END
      ELSE BEGIN
        row := GLOBAL.tit_row + 2;
        col := 42;
      END;
      lineas := lineas + 1;
    END;

  END;
END;  { Print_Msgs }

FUNCTION Add_Commas(           { Fabio }
  {+} n : LONGINT
) {>>>} : STRING;
{ RESULT
  Returns a string with 'n' re-written
  with commas each 3 digits.           }
VAR
  s : STRING [80];
  c : INTEGER;
BEGIN { Add_Commas }
  Str (n, s);           { '1,234,567,890' }
  c := Length (s) - 3;
  WHILE c > 0 DO BEGIN
    Insert (',', s, c+1);
    DEC (c, 3);
  END;
  Add_Commas := s;
END;  { Add_Commas }

PROCEDURE BOOM(          { EXPORT }     { ADH }
  {+} x,y: WORD
);
{ RESULTADO
  Despliega en la posicin (x,y) de la pantalla un
  rtulo grandote que dice BOOM! }

  PROCEDURE Wxy(x,y: WORD; s:STRING);
  BEGIN
    Crt.GotoXY(x,y);
    Write(s);
  END;

BEGIN { BOOM }
  Wxy(x, y+0, '                      ');
  Wxy(x, y+1, '                          ');
  Wxy(x, y+2, '                       ');
  Wxy(x, y+3, '                        ');
  Wxy(x, y+4, '                          ');
  Wxy(x, y+5, '                            ');
  Wxy(x, y+6, '                      ');
END;  { BOOM }

PROCEDURE WriteXY(  { LOCAL }       { MVA }
  {+} x : WORD;     { Posicin horizontal de cursor }
  {+} y : WORD;     { Posicin vertical de cursor   }
  {+} s : STRING    { Hilera a desplegar            }
);
{ RESULTADO
  Escriben en la posicin (x,y) de la pantalla la
  hilera "s".
  - Es muy cmoda para abreviar la combinacin
    GotoXY() + Write().  }
BEGIN { WriteXY }
  GotoXY(x,y);
  Write(s);
END;  { WriteXY }

FUNCTION ExistFile (     { Fabio Esquivel }
  {+} name : STRING
) {>>>>>>} : BOOLEAN;
{ RESULTADO
  Devuelve TRUE si en el disco existe un archivo
  llamado "name", y FALSE en caso contrario. }
VAR
  dir_info : SearchRec;
BEGIN { ExistFile }
  Dos.FindFirst(name, AnyFile, dir_info);
  ExistFile := (DosError = 0);
END;  { ExistFile }

PROCEDURE GoWork;
{ RESULTADO
  Este es el real programa  principal de la aplicacin
  en que se trabaja. Es quien se encarga del despliege
  y manejo del men y sus opciones.                    }
VAR
  exit_flag : BOOLEAN; { bandera para salida del men }
  i         : BYTE;    { contador de ciclos for-do    }
  ch        : CHAR;    { lee una tecla                }

BEGIN { GoWork }
  Crt.Window(0,0,24,79); { pantalla completa NO es borada }

  { se obtienen los nombres de cada opcin invocando a }
  { las funciones con la bandera de despliegue en TRUE }

  GLOBAL.on_display := TRUE;
  FOR GLOBAL.option := 'A' TO GLOBAL.last_option DO BEGIN
    GLOBAL.op_list[GLOBAL.option].op_proc;
  END;

  { llama la rutina de inicializacin }
  IF @GLOBAL.opt_Init <> NIL THEN BEGIN
    GLOBAL.option := PRED('A');
    GLOBAL.opt_Init ;
  END;

  { se llama a la opcin forzada, si existe }
  IF @GLOBAL.opt_forced <> NIL THEN BEGIN

    GLOBAL.option := 'A';
    WHILE @GLOBAL.op_list[GLOBAL.option].op_proc
          <> @GLOBAL.opt_forced
    DO BEGIN
      GLOBAL.option := SUCC(GLOBAL.option);
    END;

    IF @GLOBAL.opt_preMenu <> NIL THEN BEGIN
      GLOBAL.on_display := FALSE;
      GLOBAL.opt_preMenu;
    END;

    { ejecuta varias vece la opcin forzada }
    FOR i := 1 TO GLOBAL.force_cnt DO BEGIN

      IF @GLOBAL.opt_preOpc <> NIL THEN BEGIN
        GLOBAL.on_display := FALSE;
        GLOBAL.opt_preOpc;
      END;

      { al fin: invoca a la rutina }
      GLOBAL.on_display := FALSE;
      GLOBAL.opt_forced;

      IF @GLOBAL.opt_postOpc <> NIL THEN BEGIN
        GLOBAL.on_display := FALSE;
        GLOBAL.opt_postOpc;
      END;

    END;
  END;

  exit_flag         := FALSE;
  GLOBAL.on_display := TRUE;

  WHILE NOT exit_flag DO BEGIN

    IF GLOBAL.on_display THEN BEGIN
      { TITULO DEL MENU Y MENSAJES }

      { invoca a la rutina que se debe ejecutar }
      { antes de desplegar el men principal.   }
      IF @GLOBAL.opt_preMenu <> NIL THEN BEGIN
        GLOBAL.on_display := FALSE;
        GLOBAL.opt_preMenu;
      END;

      GotoXY(GLOBAL.tit_col,GLOBAL.tit_row);
      Write(GLOBAL.menu_tit);

      FOR i := 0 TO Ord(GLOBAL.menu_tit[0]) - 1 DO BEGIN
        GotoXY(GLOBAL.tit_col+i,GLOBAL.tit_row+1);
        Write(GLOBAL.under);
      END;

      { se imprimen los mensajes del men }
      Print_Msgs;

      GLOBAL.option     := 'A';
      GLOBAL.on_display := FALSE;
    END
    ELSE BEGIN
      GotoXY(3, 25);
      Write(
        'Memoria disponible : ',
        Add_Commas(System.MemAvail),
        ' bytes');
      GotoXY(50, 25);
      ClrEol;
      Write('Opcin : :   Esc ==> salir');
      GotoXY(58,25);

      { se repite la lectura de la opcin hasta obtener una }
      { vlida, en el rango ['A'..last_option], o hasta     }
      { o que se lea la tecla Esc.                          }
      REPEAT
        Flush_Keyboard;
        ch := UpCase(Crt.ReadKey);
        IF NOT (
          ( (ch IN ['A'..'Z']) AND (ch <= GLOBAL.last_option) )
            OR
          ( ch = Esc )
        )
        THEN BEGIN
          Beep;
        END;
      UNTIL
          ( (ch IN ['A'..'Z']) AND (ch <= GLOBAL.last_option) )
            OR
          ( ch = Esc );

      GLOBAL.option := ch;

      IF GLOBAL.option = Esc THEN BEGIN
        { hay que terminar porque se ley Esc }
        exit_flag := TRUE;
      END
      ELSE BEGIN
        GotoXY(58, 25); Write(GLOBAL.option);
        Crt.Delay(100);

        { se invoca a la rutina que debe ejecutarse }
        { antes de entrar a cada opcin del men    }
        IF @GLOBAL.opt_preOpc <> NIL THEN BEGIN
          GLOBAL.on_display := FALSE;
          GLOBAL.opt_preOpc;
        END;

        { se llama a la rutina escogida del menu }
        GLOBAL.on_display := FALSE;
        GLOBAL.op_list[GLOBAL.option].op_proc;
      END;

      { invoca a la rutina que se debe ejecutar }
      { despus de dejar cada opcin del men.  }
      GLOBAL.on_display := FALSE;
      IF (@GLOBAL.opt_postOpc <> NIL)
          AND
         (NOT exit_flag)
      THEN BEGIN
        GLOBAL.opt_postOpc;
      END;

      GLOBAL.on_display := TRUE;
    END; { IF -> on_diplay }

  END; { WHILE ->exit_flag }

  { llama la ltima rutina }
  IF @GLOBAL.opt_Done <> NIL THEN BEGIN
    GLOBAL.opt_Done;
  END;

  { se reinicializan todas la variables globales, lo que }
  { permite redefinir un nuevo men.                     }
  InitGlobal;
END;   { GoWork }

PROCEDURE KeyDelay(       { EXPORT } { Adolfo }
   {+} secDelay : INTEGER { Cunto a esperar  }
);
{ RESULTADO
  Espera hasta por secDelay segundos, o hasta
  que el usuario pulse alguno tecla.
  - Si sec > 59 entonces espera slo 59 secs. }
{ DEPURADO:                        ADH-01/88  }

TYPE
 int = INTEGER; { abreviacin }

VAR
  initSec, t1,
  nowSec      : WORD;   { se complica por no ser INTs }
  h,m,s100    : WORD;   { GetTime las necesita, yo no }
  diff        : INTEGER;
  stop        : BOOLEAN;
  ch          : CHAR;
BEGIN { KeyDelay }
  GetTime(h,m,initSec,s100);     { segundos de la hora inicial }
  IF secDelay > 59 THEN BEGIN
    secDelay := 59
  END
  ELSE IF secDelay < 0 THEN BEGIN
    secDelay := 0;
  END;

  stop := FALSE;
  diff := 0;
  WHILE NOT stop AND (diff < secDelay) DO BEGIN
    GetTime(h,m,nowSec,s100);     { segundos de la hora actual }
    stop := Crt.KeyPressed;       { salga al tocar una tecla   }
    t1 := nowSec;                 { WORD no tiene negativos    }
    diff := int(t1)-int(initSec); { tiempo que ha transcurrido }

    { agrega 60 segundo cuando da vuelta la aguja }
    IF diff < 0 THEN BEGIN
      diff := diff + 60;
    END;
  END;

  { limpia el buffer de lectura }
  Flush_Keyboard;
END; { KeyDelay }

PROCEDURE KeyDelayPlus(               { ADH }
  {?} VAR delay: WORD;   { tiempo de espera }
  {+}     delta: WORD    { fraccin de aceleracin }
);
{ RESULTADO
  Espera por "delay" segundos, o hasta que el usuario
  pulse una tecla diferente a "+" y "-" y a las
  flechas del teclado (/\ \/ -> <- PgUp PgDn)

  - Retorna inmediatamente cuando tanto "delta" como
    "delay" son cero.

  - Si "delay" es mayor de 59 segundos, entonces
    slo espera por 59 segundos.

  - Si el usuario pulsa el "+", entonces espera una
    fraccin "delta" ms de segundo.
  - Si el usuario pulsa el "-", entonces espera una
    fraccin "delta" menos de segundo.

  - Los valores vlidos para delta son los siguientes:
    - 1: Cada +/- incremente o decrementa 1 segundo
    - 2: Cada +/- incremente o decrementa 1/2 segundo
    - 3: Cada +/- incremente o decrementa 1/3 segundo
    - 4: Cada +/- incremente o decrementa 1/4 segundo
    - 5: Cada +/- incremente o decrementa 1/5 segundo
  - Si delta>5, se asume delta=5;

  - Al pulsar PgUp, KeyUp (/\) o KeyRigth (->) se
    obtiene el mismo efecto que al pulsar "+".
  - Al pulsar PgDn, KeyDn (\/) o KeyLeft (<-) se
    obtiene el mismo efecto que al pulsar "-".

  - Cuando el usuario pulsa las teclas "+" o "-"
    entonces aumenta o disminuye el valor de "delay".
    De esta manera, si se vuelve a invocar al
    procedimiento con "delay" como argumento, entonces
    el tiempo de espera ser diferente, lo que le
    permite al usuario controlar la velocidad de
    ejecucin del programa. }

CONST
  KeyUp    = 72;    KeyDn    = 80;
  PgUp     = 73;    PgDn     = 81;
  KeyRight = 77;    KeyLeft  = 75;

VAR
  now,
  init,   h,m,
  diff,   s100 : WORD;

  ch  : CHAR;
  done: BOOLEAN;
BEGIN { KeyDelayPlus }
  { se sale en el caso lmite }
  IF (delay = 0) AND (delta = 0) THEN BEGIN
    EXIT;
  END;

  { segundos de la hora inicial }
  Dos.GetTime(h,m,init,s100);

  IF delay > 59 THEN BEGIN
    delay := 59;
  END;

  IF delta = 0 THEN BEGIN
    delta := 1;
  END
  ELSE IF delta > 5 THEN BEGIN
    delta := 5;
  END;

  { diff es el tiempo que ha trasncurrido }
  { desde que se entr al procedimiento.  }
  diff := 0;

  delay := delay * delta;
  done  := FALSE;
  REPEAT
    { obtiene los segundos de la hora actual }
    Dos.GetTime(h,m,now,s100);

    IF Crt.KeyPressed THEN BEGIN
      ch := Crt.ReadKey;

      IF ch = CHR(0) THEN BEGIN
        { transforma las teclas especiales }
        ch := Crt.ReadKey;
        CASE ORD(ch) OF
          PgUp,
          KeyUp,
          KeyRight:  ch := '+';

          PgDn,
          KeyDn,
          KeyLeft :  ch := '-';
        END;
      END;

      CASE ch OF
        '+': IF delay < 59 * delta THEN BEGIN
               INC(delay);
             END;

        '-': IF delay > 0 THEN BEGIN
               DEC(delay);
             END;
      ELSE
        { slgase al tocar cualquier otra tecla }
        done := TRUE;
      END;
    END;

    { calcula el tiempo que ha transcurrido }
    IF now < init THEN BEGIN
      { agrega 60 segundo si la aguja da vuelta }
      diff := (60 + now - init) * delta;
    END
    ELSE BEGIN
      diff := (now-init) * delta;
    END;

  UNTIL done OR (diff > delay);

  { restaura el valor de delay }

  IF delay <> 1 THEN BEGIN
    { si delay estaba en cero y el usuario pulso '+',
      entonces hay que evitar que el valor final de
      delay sea cero. }
    delay := delay DIV delta;
  END;
  IF delay > 59 THEN BEGIN
    delay := 59;
  END;

  { limpia el buffer del teclado. }
  WHILE Crt.KeyPressed DO BEGIN
    ch := Crt.ReadKey;
  END;

  {$IFDEF KeyDelayPlus}
    { Codigo para probar KeyDelayPlus (delay, delta) }
    ClrScr;
    delay := 15;

    FOR i := 1 TO MaxInt DO BEGIN

      REPEAT

        GotoXY(10,10);
        ClrEol;

        FOR j := 1 TO System.Random(30) DO BEGIN
          Write('.');
          time := WhereX;
          disk := WhereY;
          { despliega el valor actual de delay }
          GotoXY(65,1);
          Write('delay ==> ', delay DIV delta:3);

          KeyDelayPlus(delay, delta);
          GotoXY(time,disk);
        END;

        ClrEol;

      UNTIL j = 0;

    END;
  {$ENDIF}
END;  { KeyDelayPlus }

PROCEDURE Beep; { EXPORT } { Gustavo A. Sanabria S. }
{ RESULTADO
  Produce un 'beep' armonioso para el oido.         }
{ NOTAS
  - Tomado de
  Steve Wood. Using TURBO PASCAL Version 4.
  (Berkeley, CA:Osborne McGraw-Hill,1988). pg. 307.
  - Si se requiere de sonidos diversos, mover las
    definiciones de Notes y el procedimiento Play
    para  hacerlos globales, y mover el encabeza-
    miento de Play a la seccin de INTERFACE.       }

TYPE
  Notes = (C,CF,D,DF,E,F,FF,G,GF,A,AF,B);
VAR
  i : INTEGER;

  PROCEDURE Play(          { LOCAL } { Alonso Sanabria }
    {+} note     : Notes;  { nota a tocar }
    {+} duration : INTEGER { duracion del sonido }
  );
  { RESULTADO
    Toca la nota "note" durante "duration" milsimas
    de segundo }
  { NOTAS
    - Tomado de [Wood88], pag. 308. }

  VAR
    frecuency : REAL;  { frecuencia de la nota }
    loop      : Notes;
  BEGIN { Play }
    frecuency := 522;                { Medio C }
    FOR loop := CF TO note DO BEGIN
      frecuency := frecuency * 1.059463094;
    END;
    Sound(Round(frecuency));    { Toca la nota }
    Delay(duration);
    NoSound;
  END;  { Play }

BEGIN { Beep }
  FOR i := 1 TO 2 DO BEGIN
    Play(G,50);
    Play(D,50);
  END;
END;  { Beep }

PROCEDURE Flush_Keyboard; { EXPORT } { Sanabria }
{ RESULTADO
  Limpia el buffer del teclado.             }
VAR
  ch : CHAR;
BEGIN { Flush_Keyboard }
  WHILE Crt.KeyPressed DO BEGIN
    ch := Crt.ReadKey;
  END;
END;  { Flush_Keyboard }

FUNCTION ReadKey: CHAR;  { EXPORT }     { ADH }
{ RESULTADO
  Lee una letra del teclado, pero ignora todos
  los cdigos de las teclas especiales, como
  last teclas Ctrl y Alt.
  - Limpia el buffer del teclado antes de hacer
    la lectura. }
VAR
  ch : CHAR;
BEGIN { ReadKey }

  WHILE Crt.KeyPressed DO BEGIN
    ch := Crt.ReadKey;
  END;

  ch := Crt.ReadKey;

  IF ch = CHR(0) THEN BEGIN
    ch := Crt.ReadKey;
  END;

  ReadKey := ch;
END;  { ReadKey }

PROCEDURE Lea_Elem(                              { Adolfo }
  {-} VAR c : CHAR;   { valor leido, pero como un carcter }
  {-} VAR i : INTEGER { valor leido, pero como un entero  }
);
{ RESULTADO
  Lee de la pantalla el siguiente valor a almacenar en
  un Elem_ADT.
  Para comodidad del operador, acepta tanto nmeros
  compuestos de varios dgitos, como una letra.
  - Si el operador digita un letra, regresa su valor
    inmediatamente en la variable "c", y el valor de
    "i" es cero.
  - Si el operador digita un nmero, lee varios dgitos
    adicionales y devuelve el nmero leido en "i", y el
    valor de "c" es CHR(0).
  - Si el valor digitado es una tecla del teclado
    extendido o ESCape, entonces regresa 0.
  Esta funcin est construida de forma que devuelva los
  datos que las operacions Elem_T.Get y Elem_T.Put
  esperan.                                                }
{ REQUIERE
  De un operador comprensivo, pues esta rutina no est
  programada de la forma ms efectiva.                    }
VAR
  salir    : BOOLEAN;
  ch       : CHAR;
  s        : STRING;
  valor,
  err      : INTEGER;
  x,y      : BYTE;
BEGIN { Lea_Elem }
  x := WhereX;
  y := WhereY;
  Write('':5);
  GotoXY(x,y);

  ch := Crt.ReadKey;

  IF ch = CHR(0) THEN BEGIN
    ch := Crt.ReadKey;
    Test.Beep;
    c := CHR(0);
    i := 0;
    EXIT;
  END;

  IF ch = CHR(27) THEN BEGIN
    c := CHR(0);
    i := 0;
    EXIT;
  END;

  IF NOT (ch IN ['0'..'9']) THEN BEGIN
    c := ch;
    i := 0;
    Write(c);
    EXIT;
  END;

  Write(ch);
  ReadLn(s);
  s := ch + s;

  Val(s, valor, err);
  IF (err <> 0) THEN BEGIN
    Test.Beep;
    c := CHR(0);
    i := 0;
  END
  ELSE BEGIN
    c := CHR(0);
    i := valor;
  END;

  WHILE Crt.KeyPressed DO BEGIN
    ch := Crt.ReadKey;
  END;
END;  { Lea_Elem }

BEGIN { Test }
  InitGlobal;
END.  { Test }

{ EOF: Test.pas }
