// FILE: test.cpp

#include "test.hpp"
#include "tools.hpp"
#include <string.h>
#include <conio.h>
#include <stdio.h>
#include <dos.h>

// Tecla Escape
#define ESC  27
#define NULL 0

struct option_rec {
  option_proc op_proc; /* rutina de la opcin  */
  str80_t     msg;     /* mensaje de la opcin */
};

struct global_rec {
  char        option;
  char 	      last_option;
  option_rec  op_list[26];
  bool	      on_display;
  option_proc opt_first;
  option_proc opt_last;
  option_proc opt_pre;
  option_proc opt_inter;
  option_proc opt_post;
  option_proc opt_forced;
  str80_t     menu_tit;
  unsigned    char tit_col;
  unsigned    char tit_row;
  unsigned    char op_col;
  int	      count;
  char        under;
};

global_rec GLOBAL;

void init_global()
/* RESULTADO
   Inicializa las variables globales */
{ // init_global
  for (int i = 0; i < 26; i++)
    GLOBAL.op_list[i].op_proc = NULL;
  GLOBAL.op_col      = 40;    /* mensajes centrados */
  GLOBAL.option      = 'A';
  GLOBAL.last_option = 64;
  GLOBAL.count       = NULL;

  GLOBAL.opt_forced = NULL;
  GLOBAL.opt_first  = NULL;
  GLOBAL.opt_last   = NULL;
  GLOBAL.opt_inter  = NULL;
  GLOBAL.opt_pre    = NULL;
  GLOBAL.opt_post   = NULL;

  GLOBAL.on_display  = TRUE;
  GLOBAL.tit_col     = 40;  /* centrado en la fila 1 */
  GLOBAL.tit_row     = 1;
  GLOBAL.under       = NO_LINE;
} // init_global

void title(
  const char*   tit,
  unsigned char y_pos,
  char 		underline
)
/* RESULTADO
   Centra y despliega el ttulo del men en la
   fila y_pos subrrayado con underline.        */
{ // title
  strcpy(GLOBAL.menu_tit, tit);
  GLOBAL.tit_col  = 40 - strlen(tit) / 2;
  GLOBAL.tit_row  = y_pos;
  GLOBAL.under    = underline;
} // title

void say(
  const char* op_msg
)
/* RESULTADO
   Guarda el mensaje para la opcin y calcula donde
   desplegar centrada la ms larga y las dems.     */
{ // say
  unsigned char center;
  strcpy(GLOBAL.op_list[GLOBAL.option].msg, op_msg);
  center = 40 - strlen(op_msg) / 2;
  if (center < GLOBAL.op_col)
    GLOBAL.op_col = center;
} // say

bool display()
/* RESULTADO
   Chequea la variable on_display para despliege del
   ttulo del men y de los mensajes de las opciones */
{ // display
  return GLOBAL.on_display;
} // display

void first_proc(
  option_proc proc
)
/* RESULTADO
   Almacena una rutina que ha de ejecutarse UNICAMENTE
   antes de entrar al men la primera vez.             */
{ // first_proc
  GLOBAL.opt_first = proc;
} // first_proc

void last_proc(
  option_proc proc
)
/* RESULTADO
   Almacena una rutina que ha de ejecutarse UNICAMENTE
   al salir del men por ultima                        */
{ // last_proc
  GLOBAL.opt_last = proc;
} // last_proc

void pre_menu(
  option_proc proc
)
/* RESULTADO
   Ejecuta la rutina proc en cada ciclo antes
   de entrar en el men                       */
{ // pre_menu
  GLOBAL.opt_pre = proc;
} // pre_menu

void inter_menu(
  option_proc proc
)
/* RESULTADO
   Ejecuta la rutina proc en cada ciclo antes
   de despus de salir del men pero antes de
   entrar a ejecutar la opcin escogida.     */
{ // inter_menu
  GLOBAL.opt_inter = proc;
} // inter_menu

void post_menu(
  option_proc proc
)
/* RESULTADO
   Ejecuta la rutina proc en cada ciclo despus
   de salir del men                            */
{ // post_menu
  GLOBAL.opt_post = proc;
} // post_menu

void force_option(
  option_proc proc,
  int         cnt
)
/* RESULTADO
   Fuerza la opcin proc para realizarse cnt veces proc
   ser la nica opcin del men  que se realizar.  Al
   hacer este  llamado NO se visualizar ningn mensaje
   que indique que se pide esa o cualquier otra opcin. */
/* REQUIERE cnt >= 1                                    */
{ // force_option
  GLOBAL.opt_forced = proc;
  GLOBAL.count      = cnt;
  GLOBAL.on_display = FALSE;
} // force_option

void add_option(
  option_proc proc
)
/* RESULTADO
   Agrega una  nueva rutina al men.  A la primera se le
   asigna la opcin 'A', a la segunda la 'B' y as hasta
   la vigsimosexta ('Z').                               */
/* REQUIERE
   La cantidad de opciones debe ser menor o igual que 26 */
{ // add_option
  GLOBAL.last_option++;
  GLOBAL.op_list[GLOBAL.last_option].op_proc = proc;
} // add_option

void print_msgs()
/* RESULTADO
   Imprime los mensajes de las opciones en dos formatos:
   centradas de  acuerdo con el mensaje ms  extenso o a
   dos columnas si son demasiadas opcines o el men est
   muy abajo en la pantalla  y se desaregla la presenta-
   cin                                                 */
{ // print_msgs
  if (23 - GLOBAL.tit_row > GLOBAL.last_option - 65) {
    /* si hay suficiente espacio se centran la opciones */
    for (GLOBAL.option = 'A'; GLOBAL.option <= GLOBAL.last_option;
	 GLOBAL.option++) {
      gotoxy(GLOBAL.op_col, GLOBAL.option - 63 + GLOBAL.tit_row);
      putchar(GLOBAL.option);
      printf(". %s",GLOBAL.op_list[GLOBAL.option].msg);
    }
  }
  else {
    /* si no hay suficiente espacio lo imprime:
       (columna 2)             (columna 42)
       A. Opcin 1             B. Opcin 2
       C. Opcin 3             D. Opcin 3
       ...
       (los mensajes son truncados si son muy largos)
    */

    int col = 2;
    int row = GLOBAL.tit_row + 2;
    for (GLOBAL.option = 'A'; GLOBAL.option <= GLOBAL.last_option;
	 GLOBAL.option++) {
      gotoxy(col,row);
      putchar(GLOBAL.option);
      printf(". %s",GLOBAL.op_list[GLOBAL.option].msg);

      if (GLOBAL.option % 2)
	row++;
      col = (col < 42) ? 42 : 2;
    }
  }
} // Print_Msgs

void gowork()
/* RESULTADO
   Este es el real programa  principal de la aplicacin
   en que se trabaja. Es quien se encarga del despliege
   y manejo del men y sus opciones.                    */
{ // gowork
  window(1,1,25,80);  /* pantalla completa NO es borada */
  if (GLOBAL.opt_forced != NULL) {
    for (int i = 0; i < GLOBAL.count; i++)
      GLOBAL.opt_forced();
  }
  else {
    /* se llama para llenar los mensajes de las opciones */
    for (GLOBAL.option = 'A'; GLOBAL.option <= GLOBAL.last_option;
	 GLOBAL.option++) {
      GLOBAL.op_list[GLOBAL.option].op_proc();
    };

    /* llama la primera rutina */
    if (GLOBAL.opt_first != NULL)
      GLOBAL.opt_first();

    bool exit_flag = FALSE;
    while (!exit_flag) {
      if (GLOBAL.on_display)  {       /* TITULO DEL MENU Y MENSAJES */
	/* llama si hay una rutina que ejecutar antes de */
	/* entrar al  men                               */
	if (GLOBAL.opt_pre != NULL)
	  GLOBAL.opt_pre();

	gotoxy(GLOBAL.tit_col,GLOBAL.tit_row);
	cprintf("%s",GLOBAL.menu_tit);

	for (int i = 0; i <= strlen(GLOBAL.menu_tit) - 1; i++) {
	  gotoxy(GLOBAL.tit_col+i,GLOBAL.tit_row+1);
	  putchar(GLOBAL.under);
	}

	/* se imprimen los mensajes del men */
	print_msgs();

	GLOBAL.on_display = FALSE;
	GLOBAL.option     = 'A';
      }
      else {
	gotoxy(50, 25);
	clreol();
	cprintf("Opcin : :   Esc ==> salir");
	gotoxy(58,25);

	/* se repite hasta que sea una opcin vlida ('A'..last_option) */
	/* o que se presione Esc.                                       */
	char ch, ch_ok;
	do {
	  clrkeybuf();
	  ch    = upcase(readkey());
	  ch_ok = (('A' <= ch) && (ch <= 'Z') && (ch <= GLOBAL.last_option))
		  || ( ch == ESC );
	  if (!ch_ok)
	    beep();
	} while (!ch_ok);
	GLOBAL.option = ch;

	if (GLOBAL.option == ESC)    /* presionar Esc para salir   */
	  exit_flag = TRUE;
	else {
	  gotoxy(58, 25); putchar(GLOBAL.option);
	  delay(100);

	  /* se ejecuta la rutina interna antes de entrar a la opcin */
	  if (GLOBAL.opt_inter != NULL)
	    GLOBAL.opt_inter();

	  /* se llama a la rutina escogida del menu */
	  GLOBAL.op_list[GLOBAL.option].op_proc();
	};

	/* llama si hay una rutina que despus antes de */
	/* salir del men                               */
	if (GLOBAL.opt_post != NULL)
	  GLOBAL.opt_post();

	GLOBAL.on_display = TRUE;
      }  /* on_display */

    }; /* exit_flag */

    /* llama la ltima rutina */
    if (GLOBAL.opt_last != NULL)
      GLOBAL.opt_last();
  }; /* op_forced */

  /* se reinicializan todas la variables globales, lo que */
  /* permite redefinir un nuevo men.                     */
  init_global();
} // gowork
